
from PyQt6 import QtCore, QtGui, QtWidgets
import os, sys, logging, shutil, shlex, tempfile
from logging.handlers import RotatingFileHandler
from typing import Tuple, List
from abstract_utilities import *



from .imports import *
class runnerTab(QWidget):
    def __init__(self, parent=None):
            super().__init__(parent)
            initFuncs(self)
            self.init_path = '/var/www/TDD/my-app'
            self.initializeInit()

            root = QVBoxLayout(self)

            # --- Top row ------------------------------------------------------------
            top = self.init_top_row_create()
            root.addLayout(top)

            # --- Issue trees + editor ----------------------------------------------
            tree_stack = self.init_tree_creation()
            dict_panel = self.init_dict_panel_creation()
            right_panel = self.init_text_editor_creation()
            editor_row  = self.init_horizontal_split(tree_stack, right_panel)
            lower_split = self.init_vertical_split_creation(dict_panel, editor_row)

            # --- Log panel (this was missing) --------------------------------------
            # Move the existing view toggles (rb_all/err/wrn, cb_try_alt_ext) above the log
            view_row = self.init_view_row_create()
            log_panel = QWidget(); log_lay = QVBoxLayout(log_panel)
            log_lay.addLayout(view_row)
            log_lay.addWidget(self.log_view, 1)

            # --- Final stack: [log panel] over [dict + trees/editor] ----------------
            main_split = self.init_vertical_split_creation(log_panel, lower_split)
            root.addWidget(main_split, 1)

            self.init_set_buttons(tree_stack)
    def start():
        startConsole(runnerTab)




# abstract_x11/window_utils.py
from ..src import *
from .monitor_utils import *
from .selection_utils import *
from .platform_utils import *
from .selection_utils import *
def classify_type(title: str) -> str:
    """Stub — replace with your real classification logic."""
    title = title.lower()
    if "terminal" in title:
        return "Terminal"
    if "code" in title:
        return "Editor"
    return "Window"

def get_windows(run_cmd, self_pid=None, self_win_hex=None) \
        -> List[Tuple[str, str, str, str, str, bool]]:
    """
    Returns [(id, pid, title, monitor, type, has_selection)]
    """
    windows = []

    mons = get_monitors(run_cmd)
    primary_owner = detect_primary_owner(run_cmd)

    out = run_cmd("wmctrl -l -p -G")

    for line in out.splitlines():
        parts = line.split()
        if len(parts) < 8:
            continue

        win_id, desktop, pid, x, y, w, h = parts[:7]
        title = " ".join(parts[8:])

        # Skip self window
        if self_pid and pid == self_pid:
            continue
        if self_win_hex and win_id.lower() == self_win_hex.lower():
            continue

        x, y = int(x), int(y)

        # Determine monitor
        monitor_name = "Unknown"
        for name, mx, my, mw, mh in mons:
            if mx <= x < mx + mw and my <= y < my + mh:
                monitor_name = name
                break

        win_type = classify_type(title)
        has_selection = (win_id.lower() == primary_owner)

        windows.append((win_id, pid, title, monitor_name, win_type, has_selection))

    return windows



# abstract_x11/selection_utils.py

from ..src import *
from .platform_utils import *
def clear_primary_selection(status_bar=None) -> None:
    """Clears PRIMARY selection on X11."""
    if is_wayland():
        if status_bar:
            status_bar.showMessage("Cannot clear selection on Wayland", 3000)
        return

    cb = QGuiApplication.clipboard()
    try:
        cb.setText("", QClipboard.Mode.Selection)
        if status_bar:
            status_bar.showMessage("Highlight cleared", 2000)
    except Exception as e:
        if status_bar:
            status_bar.showMessage(f"Error clearing selection: {e}", 3000)

def detect_primary_owner(run_cmd) -> Optional[str]:
    """
    Returns hex window id like '0x3c00007'.
    """
    try:
        out = run_cmd("xprop -root | grep PRIMARY_SELECTION")
    except Exception:
        return None

    m = re.search(r"window id # (0x[0-9a-fA-F]+)", out)
    return m.group(1).lower() if m else None



import subprocess,re
# optional regex fallback (works for simple cases of `export function foo(...)` or `export const foo = (...) =>`)
export_fn_re = re.compile(
    r"""
    ^\s*export\s+
    (?:
      (?:async\s+)?function\s+([A-Za-z0-9_$]+)\s*\(([^)]*)\) |
      const\s+([A-Za-z0-9_$]+)\s*=\s*(?:async\s*)?\(([^)]*)\)\s*=> 
    )
    """,
    re.M | re.X,
)
ROOT = "/var/www/modules/packages/abstract-apis"
INSPECT_MJS = "/var/www/modules/packages/inspect-dts.mjs"  # expects to print JSON: [{name, params:[{name,type},...]}, ...]



import pandas as pd
from sqlalchemy import create_engine, MetaData, Table, text, inspect
from sqlalchemy.orm import sessionmaker
from sqlalchemy.exc import SQLAlchemyError
from abstract_gui import startConsole
from abstract_utilities import get_env_value
from abstract_utilities.import_utils import initFuncs



from __future__ import annotations
import re,sys
from collections import deque
from dataclasses import dataclass, field
from enum import Enum, auto
from pathlib import Path
from typing import Callable, Optional





from .apiConsole import apiConsole
from .clipit import ClipIt
from .finderConsole import finderConsole
from .logConsole import logConsole
from .appRunner import appRunner
from .reactRunner import *
from .windowManager import windowManager
from .webPardner import webPardner
from .databaseViewer import databaseViewer
from .imageTab import startImageConsole, imageTab
from .dbImageViewer import startDbImageConsole,DbImageViewer
from abstract_gui.QT6 import QTabWidget,QMainWindow
from abstract_gui.QT6.utils.console_utils import ConsoleBase
from abstract_gui.QT6.utils.console_utils import startConsole

# Content Finder = the nested group you built (Find Content, Directory Map, Collect, Imports, Diff)
class ideConsole(ConsoleBase):
    def __init__(self, *, bus=None, parent=None):
        super().__init__(bus=bus, parent=parent)
        inner = QTabWidget()
        self.layout().addWidget(inner)
        # all content tabs share THIS console’s bus
        inner.addTab(reactRunner(),      "react Runner")
        inner.addTab(finderConsole(),   "Finder")
        inner.addTab(apiConsole(),   "Api")
        inner.addTab(databaseViewer(),   "databaseViewer")
        inner.addTab(ClipIt(),   "Clipit")
        inner.addTab(webPardner(),   "Web Pardner")
        inner.addTab(windowManager(),   "Window Mgr")
        inner.addTab(appRunner(),   "app runner")
        inner.addTab(imageTab(),   "Images")
        inner.addTab(DbImageViewer(),   "dbImageViewer")
        inner.addTab(logConsole(),   "logs")
        
    def start():
        startConsole(ideConsole)
        



from .imports import *
# -----------------------------------------------------------------------------
#  supervised child process runner ------------------------------------------------------------
# -----------------------------------------------------------------------------
class logConsole(QtWidgets.QWidget):
    def __init__(self, parent=None):
        try:
            super().__init__(parent)
            initFuncs(self)
            v = QtWidgets.QVBoxLayout(self)
            v.setContentsMargins(0,0,0,0)
            self.toolbar = QtWidgets.QToolBar()
            self.clear_act = self.toolbar.addAction("Clear")
            self.open_act  = self.toolbar.addAction("Open Log File")
            self.auto_scroll = QtWidgets.QCheckBox("Auto-scroll"); self.auto_scroll.setChecked(True)
            self.toolbar.addWidget(self.auto_scroll)
            v.addWidget(self.toolbar)
            self.view = QtWidgets.QPlainTextEdit(readOnly=True)
            self.view.setMaximumBlockCount(5000)  # cap memory
            v.addWidget(self.view)
            self.clear_act.triggered.connect(self.view.clear)
            self.open_act.triggered.connect(lambda: QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(log_path())))
        except Exception as e:
            print(f"{e}")
    def start():
        startConsole(logPane)




from .src import *
from .utils import *
from .logPane import *
# ---------------- supervised child process runner ----------------
class appRunnerWorker(QtCore.QObject):
    started = QtCore.pyqtSignal()
    stopped = QtCore.pyqtSignal(int)      # exit code
    crashed = QtCore.pyqtSignal(int)      # exit code

    def __init__(self, log_pane: LogPane=None, autorestart: bool = True, parent=None):
        super().__init__(parent)
        self.log = root_logger.getChild("AppRunner")
        self.p = QtCore.QProcess(self)
        self.p.setProcessChannelMode(QtCore.QProcess.ProcessChannelMode.MergedChannels)
        self.p.readyReadStandardOutput.connect(self._drain)
        self.p.readyReadStandardError.connect(self._drain)
        self.p.started.connect(lambda: (self.log.info("child started pid=%s", self.p.processId()), self.started.emit()))
        self.p.errorOccurred.connect(lambda e: self.log.error("QProcess error: %s", e.name))
        self.p.finished.connect(self._on_finished)
        self.autorestart = autorestart
        self._last_cmd = None
        self._last_cwd = None
        self._last_env = None
        self.log_pane = log_pane

    # replace your AppRunner.start with this version
    def start(self, cmd: list[str] | str, cwd: str | None = None, env: dict | None = None, force_line_buffer: bool = True):
        # Save context for possible autorestart only after successful start
        self._last_cmd = None
        self._last_cwd = None
        self._last_env = None

        pe = QtCore.QProcessEnvironment.systemEnvironment()
        pe.insert("PYTHONUNBUFFERED", "1")
        pe.insert("PYTHONFAULTHANDLER", "1")
        for k, v in (env or {}).items():
            pe.insert(k, v)

        try:
            program, args = _split_command(cmd)

            if force_line_buffer:
                program, args = _wrap_stdbuf(program, args)

            self.p.setProcessEnvironment(pe)
            if cwd:
                self.p.setWorkingDirectory(cwd)

            self.log.info("launch: %s %s (cwd=%s)", program, " ".join(map(shlex.quote, args)), cwd or os.getcwd())
            self.p.start(program, args)

            if not self.p.waitForStarted(5000):
                err = self.p.error()
                self.log.error("failed to start: %s %s (QProcess error=%s)", program, args, err.name)
                self.log_pane.append_line(f"[ERROR] failed to start: {program} {args} (QProcess error={err.name})")
                return  # don't arm autorestart on never-started process

            # only arm autorestart if we actually started
            self._last_cmd = cmd
            self._last_cwd = cwd
            self._last_env = env or {}

        except Exception as e:
            self.log.exception("start() exception while preparing command: %r", cmd)
            self.log_pane.append_line(f"[ERROR] start() exception: {e!r}")
            return


    def _drain(self):
        bs = self.p.readAllStandardOutput().data().decode(errors="replace")
        if bs:
            for line in bs.splitlines():
                root_logger.info("[child] %s", line)
                self.log_pane.append_line(line)
        bs = self.p.readAllStandardError().data().decode(errors="replace")
        if bs:
            for line in bs.splitlines():
                root_logger.error("[child:stderr] %s", line)
                self.log_pane.append_line(line)

    def _on_finished(self, code: int, status: QtCore.QProcess.ExitStatus):
        if status == QtCore.QProcess.ExitStatus.CrashExit:
            self.log.error("child crashed (code=%s)", code); self.crashed.emit(code)
        else:
            self.log.info("child exited (code=%s)", code); self.stopped.emit(code)
        if self.autorestart and self._last_cmd:
            self.log.warning("autorestart enabled; relaunching …")
            QtCore.QTimer.singleShot(1000, lambda: self.start(self._last_cmd, self._last_cwd, self._last_env))

    def stop(self):
        if self.p.state() != QtCore.QProcess.ProcessState.NotRunning:
            self.log.info("stopping child …")
            self.p.terminate()
            if not self.p.waitForFinished(3000):
                self.log.warning("terminate timed out; killing")
                self.p.kill()




from PyQt6.QtWidgets import (
    QWidget, QHBoxLayout, QVBoxLayout, QLineEdit, QSpinBox,
    QPushButton, QLabel, QScrollArea, QCheckBox, QSizePolicy,
    QFrame, QGridLayout,
)
from PyQt6.QtCore import Qt, QRunnable, QObject, pyqtSignal, QThreadPool

from ..src.utils import get_user_page, get_page_tags, USER
from ..src.constants import FILTERS


class _TagFetchSignals(QObject):
    done = pyqtSignal(list)
    error = pyqtSignal(str)


class _TagFetcher(QRunnable):
    def __init__(self, page: int, user: str | None):
        super().__init__()
        self.page = page
        self.user = user
        self.signals = _TagFetchSignals()

    def run(self):
        try:
            tags = get_page_tags(self.page, self.user)
            self.signals.done.emit(tags)
        except Exception as e:
            self.signals.error.emit(str(e))




class TagSelector(QWidget):
    COLUMNS = 4

    def __init__(self, active_filters: list[str], parent=None):
        super().__init__(parent)
        self._active: set[str] = set(active_filters)
        self._checkboxes: dict[str, QCheckBox] = {}

        root = QVBoxLayout(self)
        root.setContentsMargins(0, 0, 0, 0)
        root.setSpacing(2)

        # ── single header line ────────────────────────────────
        header = QHBoxLayout()

        self._toggle_btn = QPushButton("▶ Tags")
        self._toggle_btn.setCheckable(True)
        self._toggle_btn.setFlat(True)
        self._toggle_btn.setFixedWidth(70)
        self._toggle_btn.toggled.connect(self._toggle)

        self._status = QLabel("")
        self._status.setStyleSheet("color: gray; font-size: 10px;")

        self._add_field = QLineEdit()
        self._add_field.setPlaceholderText("Add tag…")
        self._add_field.setFixedWidth(90)
        self._add_field.returnPressed.connect(self._add_manual)

        add_btn = QPushButton("+")
        add_btn.setFixedWidth(22)
        add_btn.clicked.connect(self._add_manual)

        self._fetch_btn = QPushButton("Fetch")
        self._fetch_btn.setFixedWidth(44)
        self._fetch_btn.clicked.connect(self._request_fetch)

        header.addWidget(self._toggle_btn)
        header.addWidget(self._status, stretch=1)
        header.addWidget(self._add_field)
        header.addWidget(add_btn)
        header.addWidget(self._fetch_btn)
        root.addLayout(header)

        # ── collapsible grid ──────────────────────────────────
        self._panel = QWidget()
        self._panel.setVisible(False)

        panel_layout = QVBoxLayout(self._panel)
        panel_layout.setContentsMargins(0, 0, 0, 0)

        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        scroll.setFrameShape(QFrame.Shape.NoFrame)
        scroll.setMaximumHeight(110)

        self._box_container = QWidget()
        self._box_layout = QGridLayout(self._box_container)
        self._box_layout.setContentsMargins(2, 2, 2, 2)
        self._box_layout.setHorizontalSpacing(8)
        self._box_layout.setVerticalSpacing(1)

        scroll.setWidget(self._box_container)
        panel_layout.addWidget(scroll)
        root.addWidget(self._panel)

        self._get_context = lambda: (1, None)

        for f in sorted(self._active):
            self._add_checkbox(f, checked=True)

        self._refresh_status()

    # ── public ────────────────────────────────────────────────

    def set_context_provider(self, fn):
        self._get_context = fn

    @property
    def active_filters(self) -> list[str]:
        return [tag for tag, cb in self._checkboxes.items() if cb.isChecked()]

    # ── internals ─────────────────────────────────────────────

    def _toggle(self, checked: bool):
        self._toggle_btn.setText(f"{'▼' if checked else '▶'} Tags")
        self._panel.setVisible(checked)

    def _refresh_status(self):
        active = sum(1 for cb in self._checkboxes.values() if cb.isChecked())
        total = len(self._checkboxes)
        self._status.setText(f"{active}/{total}" if total else "")

    def _add_checkbox(self, tag: str, checked: bool = False):
        if tag in self._checkboxes:
            return
        cb = QCheckBox(tag)
        cb.setChecked(checked)
        cb.toggled.connect(lambda _: self._refresh_status())
        idx = len(self._checkboxes)
        row, col = divmod(idx, self.COLUMNS)
        self._box_layout.addWidget(cb, row, col)
        self._checkboxes[tag] = cb
        self._refresh_status()

    def _add_manual(self):
        tag = self._add_field.text().strip()
        if not tag:
            return
        self._add_checkbox(tag, checked=True)
        self._add_field.clear()
        if not self._panel.isVisible():
            self._toggle_btn.setChecked(True)

    def _request_fetch(self):
        page, user = self._get_context()
        self._fetch_btn.setEnabled(False)
        self._status.setText("…")
        fetcher = _TagFetcher(page, user)
        fetcher.signals.done.connect(self._on_tags_fetched)
        fetcher.signals.error.connect(self._on_fetch_error)
        QThreadPool.globalInstance().start(fetcher)

    def _on_tags_fetched(self, tags: list[str]):
        for tag in tags:
            self._add_checkbox(tag, checked=(tag in self._active))
        self._refresh_status()
        self._fetch_btn.setEnabled(True)
        if not self._panel.isVisible():
            self._toggle_btn.setChecked(True)

    def _on_fetch_error(self, msg: str):
        self._status.setText(f"err: {msg}")
        self._fetch_btn.setEnabled(True)
class QueryBar(QWidget):
    def __init__(self, browser):
        super().__init__()
        self.browser = browser

        root = QVBoxLayout(self)
        root.setSpacing(6)

        # ── query row ──────────────────────────────────────────
        query_row = QHBoxLayout()

        self.user = QLineEdit()
        self.user.setPlaceholderText(f"User (default: {USER})")
        self.user.textChanged.connect(self._update_preview)

        self.start_page = QSpinBox()
        self.start_page.setMinimum(1)
        self.start_page.setMaximum(99999)
        self.start_page.setPrefix("From ")
        self.start_page.valueChanged.connect(self._update_preview)

        self.end_page = QSpinBox()
        self.end_page.setMinimum(1)
        self.end_page.setMaximum(99999)
        self.end_page.setPrefix("To ")

        load_btn = QPushButton("Load")
        load_btn.clicked.connect(self.load)

        query_row.addWidget(self.user)
        query_row.addWidget(self.start_page)
        query_row.addWidget(self.end_page)
        query_row.addWidget(load_btn)

        # ── url preview ────────────────────────────────────────
        self._url_label = QLabel()
        self._url_label.setStyleSheet("color: gray; font-size: 10px;")
        self._url_label.setTextInteractionFlags(
            Qt.TextInteractionFlag.TextSelectableByMouse
        )
        self._url_label.setWordWrap(True)

        # ── tag selector ───────────────────────────────────────
        self._tag_selector = TagSelector(FILTERS)
        self._tag_selector.set_context_provider(self._current_context)

        root.addLayout(query_row)
        root.addWidget(self._url_label)
        root.addWidget(self._tag_selector)
        # in QueryBar.__init__, add to query_row:
        self._skip_downloaded = QCheckBox("Hide downloaded")
        query_row.addWidget(self._skip_downloaded)


        self._update_preview()

    # ── slots ─────────────────────────────────────────────────

    def _current_context(self) -> tuple[int, str | None]:
        user = self.user.text().strip() or None
        return self.start_page.value(), user

    def _update_preview(self):
        user = self.user.text().strip() or USER
        page = self.start_page.value()
        self._url_label.setText(f"URL: {get_user_page(user=user, page=str(page))}")


    # update load():
    def load(self):
        start = self.start_page.value()
        end = max(self.end_page.value(), start)

        FILTERS.clear()
        FILTERS.extend(self._tag_selector.active_filters)

        user = self.user.text().strip() or None
        self.browser.load_pages(
            start, end, user,
            skip_downloaded=self._skip_downloaded.isChecked(),
        )



from PyQt6.QtCore import QObject, QRunnable, pyqtSignal
from ..parser.article_parser import parse_articles
from ..src.utils import get_user_page, USER


class PageLoaderSignals(QObject):
    page_loaded = pyqtSignal(list)
    error = pyqtSignal(str)


class PageLoader(QRunnable):
    def __init__(self, page: int, user: str | None = None, skip_downloaded: bool = False):
        super().__init__()
        self.page = page
        self.user = user or USER
        self.skip_downloaded = skip_downloaded
        self.signals = PageLoaderSignals()

    def run(self):
        try:
            url = get_user_page(user=self.user, page=str(self.page))
            articles = parse_articles(url, skip_downloaded=self.skip_downloaded)
            self.signals.page_loaded.emit(articles)
        except Exception as e:
            self.signals.error.emit(str(e))



from dataclasses import dataclass
from typing import Optional, Sequence

@dataclass
class ArticleQuery:
    user: Optional[str] = None
    page: int = 1
    media_type: Optional[str] = None
    required_tags: Sequence[str] | None = None
    raw_url: Optional[str] = None



import os
import subprocess
import shlex

def resolve_directory_input(path: str) -> str:
    """
    Resolve a directory input that may be:
      - local path
      - symlink
      - GVFS-mounted SFTP
      - sftp:// URL (auto-mount via gio)

    Returns a normalized local directory path.
    Raises ValueError if it cannot be resolved.
    """
    if not path:
        raise ValueError("Empty directory path")

    path = path.strip()

    # Case 1: already a valid local directory
    expanded = os.path.realpath(os.path.expanduser(path))
    if os.path.isdir(expanded):
        return expanded

    # Case 2: sftp:// URL → try GVFS mount
    if path.startswith("sftp://"):
        try:
            subprocess.run(
                ["gio", "mount", path],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
                check=False,
            )

            # Ask gio where it mounted it
            proc = subprocess.run(
                ["gio", "info", path],
                capture_output=True,
                text=True,
            )

            for line in proc.stdout.splitlines():
                if "local path:" in line:
                    local_path = line.split("local path:", 1)[1].strip()
                    local_path = os.path.realpath(local_path)
                    if os.path.isdir(local_path):
                        return local_path

        except Exception as e:
            raise ValueError(f"Failed to resolve SFTP path: {path}") from e

    raise ValueError(f"Directory could not be resolved: {path}")



from .imports import *
def _on_variable_clicked(self, var_name: str):
    self.current_var = var_name
    self.variableSelected.emit(var_name)
    self._render_var_lists_for(var_name)
    
# --- shared helpers -----------------------------------------------------------
def _render_var_lists_for(self, var_name: str=None):
   var_name = var_name or self.current_var
   self._render_symbol_lists_for(var_name, self.var_map,
                             self.exporters_list, self.importers_list,
                             getattr(self, "var_filter_mode", "io"))
# --- variables: mirror functions with tiny wrapper(s) -------------------------
def _on_var_filter_mode_changed(self):
    self.var_filter_mode = "source" if self.rb_var_source.isChecked() else ("all" if self.rb_var_all.isChecked() else "io")
    if getattr(self, "current_var", None):
        self._render_var_lists_for(self.current_var)
        
def _add_var_button(self, name: str):
    btn = QPushButton(name)
    btn.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
    btn.clicked.connect(lambda _, n=name: self._on_variable_clicked(n))
    self.var_layout.addWidget(btn)

def _clear_var_buttons(self):
    while self.var_layout.count():
        it = self.var_layout.takeAt(0)
        w = it.widget()
        if w: w.deleteLater()

def _rebuild_var_buttons(self, names_iterable):
    self._clear_var_buttons()
    names = sorted(n for n in names_iterable if n and n != '<reexport>')
    for name in names:
        self._add_var_button(name)

def _filter_var_buttons(self, text: str):
    t = (text or '').strip().lower()
    if not getattr(self, 'var_map', None):
        return
    if not t:
        self._rebuild_var_buttons(self.var_map.keys())
    else:
        match = [n for n in self.var_map.keys() if t in n.lower()]
        self._rebuild_var_buttons(match)



from bs4 import BeautifulSoup
from dataclasses import dataclass
from ..src import get_soup, get_filtered_articles,BASE_URL

@dataclass
class ArticleData:
    title: str
    image_url: str
    article_url: str
    author: str
    author_url: str
    size: str
    pages: str
    views: str
    download_url: str


def _file_id_from_url(url: str) -> str | None:
    try:
        return url.split(BASE_URL)[1].split('/')[0]
    except (IndexError, AttributeError):
        return None


def parse_articles(url, skip_downloaded: bool = False) -> list[ArticleData]:
    registry = None
    if skip_downloaded:
        from ..managers.db import DownloadRegistry
        from ..src.constants import DB_DSN
        registry = DownloadRegistry(DB_DSN)  # returns the same instance every time


    articles = []
    for art in get_filtered_articles(url):
        try:
            download_url = art.select_one(".kslinks")["href"]

            if registry:
                file_id = _file_id_from_url(download_url)
                if file_id and registry.has(file_id):
                    continue

            articles.append(ArticleData(
                title=art.select_one(".header-title").text.strip(),
                image_url=art.select_one("img")["src"],
                article_url=art.select_one("a")["href"],
                author=art.select_one(".author a").text,
                author_url=art.select_one(".author a")["href"],
                size=art.select_one(".size").text,
                pages=art.select_one(".pages").text,
                views=art.select_one(".views").text,
                download_url=download_url,
            ))
        except (TypeError, KeyError):
            continue

    return articles



from abstract_gui.QT6 import *
from abstract_gui.QT6.utils.log_utils import *
from PyQt6.QtCore import QObject, pyqtSignal, QTimer, qInstallMessageHandler, QtMsgType, Qt
from PyQt6.QtWidgets import (
    QWidget, QVBoxLayout, QPlainTextEdit, QTabWidget, QPushButton, QHBoxLayout
)



from .src import *
from .TreeNode import *




@dataclass
class FileEntry:
    """Schema for a file in the registry."""

    relative_path: Path
    absolute_path: Path
    content: str = ""
    is_dirty: bool = False
    exists_on_disk: bool = False


@dataclass
class Config:
    """Explicit configuration schema."""

    root_path: Optional[Path] = None
    tree_indent_chars: tuple[str, ...] = ("│", "├", "└", "─", "    ", "   ")
    file_extensions: tuple[str, ...] = (".py", ".md", ".txt", ".json", ".yaml", ".yml", ".toml")



"""Robust, drop-in helpers for filesystem path inputs (Qt).

- Works with your `abstract_gui` wrapper, with graceful fallback to PyQt5/PySide6.
- Two entry points:
    - `make_path_input(...) -> QLineEdit`
    - `make_path_input_with_browse(...) -> QWidget` (composite: [QLineEdit | Browse…])
- Extras:
    - `require_package_json(path) -> (ok, msg)` validator
    - `set_project_path(line_edit, new_path, fire_handler=False, handler=None)` safe setter
    - `add_fs_completer(line_edit, root="/")` for filesystem autocompletion

Usage example (inside your class):

    from path_inputs import make_path_input, require_package_json, add_fs_completer

    def _on_path_changed(self, text: str):
        self.init_path = text

    self.path_in = make_path_input(
        init_value=getattr(self, "init_path", ""),
        on_change=self._on_path_changed,
        user_only=True,
        parent=self,
        placeholder="Project path (folder with package.json)",
        validate=require_package_json,
    )
    add_fs_completer(self.path_in)
    self.layout().addWidget(self.path_in)

"""
from __future__ import annotations
from pathlib import Path
from typing import Callable, Optional, Tuple

# --- Qt imports: prefer your wrapper, gracefully fall back ---
try:  # your wrapper (preferred)
    from abstract_gui import *  # noqa: F401,F403
except Exception:

        from PySide6.QtCore import Qt, QSignalBlocker
        from PySide6.QtGui import QPalette, QColor
        from PySide6.QtWidgets import (
            QWidget, QLineEdit, QPushButton, QHBoxLayout, QFileDialog,
            QApplication, QCompleter, QFileSystemModel,
        )

__all__ = [
    "make_path_input",
    "make_path_input_with_browse",
    "require_package_json",
    "set_project_path",
    "add_fs_completer",
]


def _apply_validation_feedback(line_edit: QLineEdit, ok: bool, msg: Optional[str] = None) -> None:
    """Clean, non-destructive validity feedback with proper colors."""
    line_edit.setToolTip("" if ok else (msg or "Invalid path"))

    if ok:
        line_edit.setStyleSheet("")
        line_edit.setPalette(QApplication.palette())  # Reset to theme
    else:
        line_edit.setStyleSheet("""
            QLineEdit {
                background-color: #ffebee;
                border: 1.5px solid #e57373;
                border-radius: 4px;
                padding: 3px;
            }
        """)
def make_path_input(
    init_value: str = "",
    on_change: Optional[Callable[[str], None]] = None,
    user_only: bool = True,  # True => textEdited; False => textChanged
    parent: Optional[QWidget] = None,
    placeholder: str = "Project path (folder with package.json)",
    validate: Optional[Callable[[str], Tuple[bool, Optional[str]]]] = None,
) -> QLineEdit:
    """
    Create a path QLineEdit with optional validation and change callback.

    Args:
        init_value: initial text
        on_change: called with normalized text whenever it changes
        user_only: if True, connect textEdited (user typing only). If False, textChanged
        parent: QWidget parent
        placeholder: placeholder text
        validate: callable(text) -> (ok, message). If provided, UI shows feedback

    Returns:
        QLineEdit
    """
    le = QLineEdit(parent)
    if init_value:
        le.setText(init_value)
    le.setPlaceholderText(placeholder)

    signal = le.textEdited if user_only else le.textChanged
    logger.info(on_change)
    if on_change is not None or validate is not None:
        def _slot(text: str) -> None:
            text = text.strip()
            if validate is not None:
                ok, msg = validate(text)
                _apply_validation_feedback(le, ok, msg)
            if on_change is not None:
                on_change(text)
        signal.connect(_slot)

    return le


def make_path_input_with_browse(
    init_value: str = "",
    on_change: Optional[Callable[[str], None]] = None,
    user_only: bool = True,
    parent: Optional[QWidget] = None,
    placeholder: str = "Project path (folder with package.json)",
    validate: Optional[Callable[[str], Tuple[bool, Optional[str]]]] = None,
) -> QWidget:
    """Composite widget: [ QLineEdit | Browse… ] with same behavior as make_path_input."""
    w = QWidget(parent)
    layout = QHBoxLayout(w)
    layout.setContentsMargins(0, 0, 0, 0)

    le = make_path_input(init_value, on_change, user_only, w, placeholder, validate)
    btn = QPushButton("Browse…", w)

    def _browse() -> None:
        start = le.text() or str(Path.home())
        d = QFileDialog.getExistingDirectory(w, "Select Project Directory", start)
        if d:
            d = str(Path(d).expanduser())
            if user_only:
                # textEdited won’t fire for setText; call handler manually if provided
                try:
                    blocker = QSignalBlocker(le)
                    le.setText(d)
                finally:
                    del blocker
                if on_change:
                    on_change(d)
            else:
                # textChanged will emit and hit the handler
                le.setText(d)

    btn.clicked.connect(_browse)
    layout.addWidget(le, 1)
    layout.addWidget(btn, 0)

    # expose the line edit handle for callers
    w.line_edit = le  # type: ignore[attr-defined]
    return w


def require_package_json(path_str: str) -> Tuple[bool, Optional[str]]:
    """Validator: path must be an existing directory containing package.json."""
    p = Path(path_str).expanduser()
    if not p.exists():
        return False, "Path does not exist"
    if not p.is_dir():
        return False, "Not a directory"
    if not (p / "package.json").exists():
        return False, "package.json not found in directory"
    return True, None


def set_project_path(
    line_edit: QLineEdit,
    new_path: str,
    *,
    fire_handler: bool = False,
    handler: Optional[Callable[[str], None]] = None,
) -> None:
    """Programmatically set the path safely.

    If the widget is connected via textEdited (user_only=True), programmatic setText
    does not emit; use fire_handler=True or call your handler manually.
    """
    new_path = str(Path(new_path).expanduser())
    try:
        blocker = QSignalBlocker(line_edit)
        line_edit.setText(new_path)
    finally:
        try:
            del blocker
        except Exception:
            pass
    if fire_handler and handler is not None:
        handler(new_path)


def add_fs_completer(
    line_edit: QLineEdit,
    root: str = "/",
    *,
    case_insensitive: bool = True,
    contains_match: bool = False,
) -> QCompleter:
    """Attach a filesystem completer to the line edit and return it (Qt5/Qt6 safe)."""
    model = QFileSystemModel(line_edit)
    model.setRootPath(root)
    comp = QCompleter(model, line_edit)

    # --- Qt5/Qt6 enum compatibility ---
    # Case sensitivity
    try:
        # Qt6 style: Qt.CaseSensitivity.CaseInsensitive
        CS = Qt.CaseSensitivity  # type: ignore[attr-defined]
        CASE_INS = CS.CaseInsensitive
        CASE_SENS = CS.CaseSensitive
    except AttributeError:
        # Qt5 style: Qt.CaseInsensitive, Qt.CaseSensitive
        CASE_INS = getattr(Qt, "CaseInsensitive", 0)
        CASE_SENS = getattr(Qt, "CaseSensitive", 1)

    comp.setCaseSensitivity(CASE_INS if case_insensitive else CASE_SENS)

    # Match mode (only applied when requested)
    if contains_match:
        try:
            MF = Qt.MatchFlag  # Qt6
            match_contains = MF.MatchContains
        except AttributeError:
            match_contains = getattr(Qt, "MatchContains", 0)  # Qt5
        comp.setFilterMode(match_contains)

    line_edit.setCompleter(comp)
    return comp




import sys
from PyQt6.QtWidgets import QApplication
from abstract_database import DatabaseBrowser
from gui.main_window import MainWindow
from gui.controller import DBController

app = QApplication(sys.argv)

browser = DatabaseBrowser(dbUrl='postgresql://solcatcher:solcatcher123!!!456@23.126.105.154:5432/solcatcher')
ui = MainWindow()
controller = DBController(ui, browser)

ui.show()
sys.exit(app.exec())



from .imports import *
from .diffParserTab import diffParserTab
from .directoryMapTab import directoryMapTab
from .extractImportsTab import extractImportsTab
from .finderTab import finderTab
from .collectFilesTab import collectFilesTab
from .scaffolder import Scaffolder
from .main import finderConsole
from .mani import ManifestViewerWindow
def startFinderConsole():
    startConsole(finderConsole)
def startManifestViewerWindow():
    startConsole(ManifestViewerWindow)



from PyQt6 import QtWidgets, QtGui, QtCore, QtGui, QtWidgets
from PyQt6.QtGui import QGuiApplication, QClipboard
from PyQt6.QtWidgets import QMessageBox,QApplication,QTextEdit,QMainWindow
from PyQt6.QtCore import Qt



from .imports import *


def _clear_files_tree(self):
    self.files_list.clear()

def _add_file_row(self, path: str, apply_checked: bool = True, overwrite_checked: bool = False):
    it = QTreeWidgetItem(self.files_list)
    it.setText(0, path)
    it.setFlags(it.flags() | Qt.ItemFlag.ItemIsUserCheckable | Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled)
    it.setCheckState(1, Qt.CheckState.Checked if apply_checked else Qt.CheckState.Unchecked)
    it.setCheckState(2, Qt.CheckState.Checked if overwrite_checked else Qt.CheckState.Unchecked)
    it.setData(0, Qt.ItemDataRole.UserRole, path)

def _fill_files_tree(self, files: list[str], *, default_apply=True, default_overwrite=False):
    self._clear_files_tree()
    if not files:
        return
    self.files_list.setUpdatesEnabled(False)
    for fp in files:
        self._add_file_row(fp, apply_checked=default_apply, overwrite_checked=default_overwrite)
    self.files_list.setUpdatesEnabled(True)

def _collect_checked_files(self) -> tuple[list[str], list[str]]:
    apply_list, overwrite_list = [], []
    for i in range(self.files_list.topLevelItemCount()):
        it = self.files_list.topLevelItem(i)
        path = it.data(0, Qt.ItemDataRole.UserRole) or it.text(0)
        if it.checkState(1) == Qt.CheckState.Checked:
            apply_list.append(path)
        if it.checkState(2) == Qt.CheckState.Checked:
            overwrite_list.append(path)
    return apply_list, overwrite_list


def get_files(self) -> list[str]:
    params = make_params(self)
    dirs, files = get_files_and_dirs(**params)
    return files



import sys
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Callable
from queue import Queue
from threading import Thread
from enum import Enum, auto
import time



import os
# ---------- path helper ----------
def get_new_image_path(input_image, basename, func=None, dir_name=None, **kwargs):
    """
    Build an output path under <input dir>/<dir_name or 'new_pics'>/<basename>,
    optionally run func(input_path, output_path, **kwargs).
    """
    dir_name = dir_name or 'new_pics'
    dirname = os.path.dirname(input_image)
    out_dir = os.path.join(dirname, dir_name)
    os.makedirs(out_dir, exist_ok=True)
    output_image = os.path.join(out_dir, basename)
    if func:
        func(input_image, output_image, **kwargs)
    return output_image



from .src import *
class NodeType(Enum):
    DIRECTORY = auto()
    FILE = auto()
@dataclass
class TreeNode:
    """Schema for a parsed tree node."""
    TreeNode=[]
    name: str
    node_type: NodeType
    depth: int
    children: list[TreeNode] = field(default_factory=list)
    parent: Optional[TreeNode] = None

    @property
    def is_file(self) -> bool:
        return self.node_type == NodeType.FILE

    @property
    def is_directory(self) -> bool:
        return self.node_type == NodeType.DIRECTORY
